﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Drawing2D;
using System.Runtime.InteropServices;

namespace CHRMapper
{
    public partial class Editor : Form
    {
        ToolTip tt = new ToolTip();
        String map;
        String Openfile;
        byte[,] MapTiles;
        byte Brush = 0;
        Bitmap[] Tile = new Bitmap[256];
        Boolean isPainting = false;
        Graphics MapGraphics;
        public Editor()
        {
            InitializeComponent();
            pictureBox2.Image = new Bitmap(32, 32);
        }

        private void LoadTiles(string map)
        {
            Char TSet = map[3];
            if (TSet == '2')
                TSet = '0';
            else if (TSet == '9' || TSet == 'A')
                TSet = '8';
            Tile = Mapper.LoadTileset(TSet);
            pictureBox3.Image = Mapper.RenderMap(Mapper.PaletteMap(), Tile);
        }
        private void LoadMap()
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Title = "Choose file to open";
            dlg.Filter = "MAP file|*.map|All files (*.*)|*.*";
            dlg.CheckFileExists = true;
            dlg.CheckPathExists = true;
            if (dlg.ShowDialog() == DialogResult.OK)
                LoadMap(dlg.FileName);
        }
        private void LoadMap(string mapfile)
        {
            byte[] Data = File.ReadAllBytes(mapfile);
            int x = (int)(Data[0]), y = (int)(Data[2]);
            MapTiles = new byte[x, y];
            for (int yi = 0; yi < y; yi++)
                for (int xi = 0; xi < x; xi++)
                    MapTiles[xi, yi] = Data[yi*x+xi+12];
            map = mapfile.Substring(mapfile.LastIndexOf('\\') + 1);
            LoadTiles(map);
            pictureBox1.Image = new Bitmap(x * 16, y * 16);
            RenderMap();
            this.Text = map + ": " + x + "x" + y;
            pictureBox1.Size = pictureBox1.Image.Size;
            Openfile = mapfile;
            SetBrush(Brush);
            MapGraphics = Graphics.FromImage(pictureBox1.Image);
            MapGraphics.CompositingMode = CompositingMode.SourceCopy;
            MapGraphics.CompositingQuality = CompositingQuality.HighSpeed;
        }
        private void RenderMap()
        {
            pictureBox1.Image = Mapper.RenderMap(MapTiles, Tile);
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            LoadMap();
            if (pictureBox1.Image == null)
                this.Close();
            Form1_SizeChanged(this, e);
        }

        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.X >= 0 && e.X < pictureBox1.Image.Width && e.Y >= 0 && e.Y < pictureBox1.Image.Height)
            {
                this.Text = map + ": " + MapTiles.GetLength(0) + "x" + MapTiles.GetLength(1) + " (" + (e.X >> 4) + ":" + (e.Y >> 4) + ") = " + MapTiles[e.X >> 4, e.Y >> 4];
                if (isPainting)
                {
                    ReplaceTile(e.X, e.Y);
                    pictureBox1.Refresh();
                }
            }
        }

        private void Form1_SizeChanged(object sender, EventArgs e)
        {
            panel1.Size = new Size(this.Width - 8 - panel2.Width, this.Height - 34);
            panel2.Location = new Point(panel1.Location.X + panel1.Width, 0);
            panel2.Height = panel1.Height;
        }

        private void pictureBox1_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.X >= 0 && e.X < pictureBox1.Image.Width && e.Y >= 0 && e.Y < pictureBox1.Image.Height)
            {
                if (e.Button == MouseButtons.Left)
                    ReplaceTile(e.X, e.Y);
                if (e.Button == MouseButtons.Right)
                    SetBrush(MapTiles[e.X >> 4, e.Y >> 4]);
            }
            pictureBox1.Refresh();
        }
        private void ReplaceTile(int x, int y)
        {
            MapTiles[x >> 4, y >> 4] = Brush;
            MapGraphics.DrawImage(Tile[Brush], x & 0xFF0, y & 0xFF0, 16, 16);
        }

        private void SaveMap()
        {
            byte[] WriteBuffer = new byte[MapTiles.GetLength(0) * MapTiles.GetLength(1) + 12];
            WriteBuffer[0] = (byte)MapTiles.GetLength(0);
            WriteBuffer[2] = (byte)MapTiles.GetLength(1);
            for (int i = 0; i < 8; i++)
                WriteBuffer[4 + i] = MapTiles[i, 0];
            List<byte> Bytes = new List<byte>();
            for (int y = 0; y < MapTiles.GetLength(1); y++)
                for (int x = 0; x < MapTiles.GetLength(0); x++)
                    Bytes.Add(MapTiles[x, y]);
            for (int i = 0; i < Bytes.Count; i++)
                WriteBuffer[i + 12] = Bytes[i];
            File.WriteAllBytes(Openfile, WriteBuffer);
            MessageBox.Show("Map saved to file!");
        }
        private void ExportBitmap()
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Title = "Where do you want to save the image to?";
            dlg.CheckPathExists = true;
            dlg.FileName = map.Substring(0, map.LastIndexOf('.'));
            dlg.Filter = "Portable Network Graphics|*.png";
            if (dlg.ShowDialog() == DialogResult.OK)
                this.pictureBox1.Image.Save(dlg.FileName, System.Drawing.Imaging.ImageFormat.Png);
            dlg.Dispose();
        }

        private void Editor_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Modifiers == Keys.Control)
            {
                switch (e.KeyCode)
                {
                    case Keys.S:
                        SaveMap();
                        break;
                    case Keys.O:
                        LoadMap();
                        break;
                    default:
                        break;
                }
            }
            else if (e.Modifiers == (Keys.Shift | Keys.Control))
            {
                switch (e.KeyCode)
                {
                    case Keys.S:
                        ExportBitmap();
                        break;
                    default:
                        break;
                }
            }
            else
            {
                switch (e.KeyCode)
                {
                    case Keys.Enter:
                        if (this.WindowState == FormWindowState.Maximized)
                            this.WindowState = FormWindowState.Normal;
                        else this.WindowState = FormWindowState.Maximized;
                        break;
                    case Keys.Space:
                        //ChangeBrush();
                        break;
                    default:
                        break;
                }
            }
        }

        private void pictureBox1_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button==MouseButtons.Left)
            isPainting = true;
        }

        private void pictureBox1_MouseLeave(object sender, EventArgs e)
        {
            isPainting = false;
        }

        private void pictureBox1_MouseUp(object sender, MouseEventArgs e)
        {
            isPainting = false;
        }

        private void pictureBox3_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.X >= 0 && e.X < pictureBox3.Image.Width && e.Y >= 0 && e.Y < pictureBox3.Image.Height)
                SetBrush((e.X >> 4) + (e.Y >> 4) * 16);
        }
        private void SetBrush(int TID)
        {
            Brush = (byte)TID;
            Bitmap NewThumb = new Bitmap(32, 32);
            for (int i = 0; i < 16; i++)
                for (int j = 0; j < 16; j++)
                    foreach (Point p in new Point[]{new Point(0,0), new Point(1,0), new Point(0,1), new Point(1,1)})
                {
                    NewThumb.SetPixel((i << 1)+p.X, (j << 1)+p.Y, Tile[Brush].GetPixel(i,j));
                }
/*            Graphics g = Graphics.FromImage(NewThumb);
            g.InterpolationMode = InterpolationMode.NearestNeighbor;
            g.DrawImage(Tile[Brush], 0, 0, 32, 32);
            g.Dispose();*/
            pictureBox2.Image = NewThumb;
        }

        private void pictureBox1_MouseDoubleClick_1(object sender, MouseEventArgs e)
        {
            if (e.Button==MouseButtons.Left && e.X >= 0 && e.X < pictureBox1.Image.Width && e.Y >= 0 && e.Y < pictureBox1.Image.Height)
            {
                int x = e.X - (e.X & 0xF);
                int y = e.Y - (e.Y & 0xF);
                int xl = x+17;
                int yl = y + 17;
                x -= 16;
                y -= 16;
                if (x < 0) x = 0;
                if (y < 0) y = 0;
                if (xl > pictureBox1.Image.Width) xl = pictureBox1.Image.Width;
                if (yl > pictureBox1.Image.Height) yl = pictureBox1.Image.Height;
                for (; x < xl; x += 16)
                    for (int yi = y; yi < yl; yi += 16)
                        ReplaceTile(x, yi);
                pictureBox1.Refresh();
            }
        }
    }
}