﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace CHRMapper
{
    public static class Mapper
    {
        public static byte[,] PaletteMap()
        {
            byte[,] Index = new byte[16, 16];
            for (int y = 0; y < 16; y++)
                for (int x = 0; x < 16; x++)
                    Index[x, y] = (byte)(y * 16 + x);
            return Index;
        }
        static string[,] Map_Tileset_Bindings = new string[11, 2] {
        { "map0.map", "0"},
        { "map1.map", "1"},
        { "map2.map", "0"},
        { "map3.map", "3"},
        { "map4.map", "4"},
        { "map5.map", "5"},
        { "map6.map", "6"},
        { "map7.map", "7"},
        { "map8.map", "8"},
        { "map9.map", "8"},
        { "mapA.map", "8"},
        };
        static public Bitmap[] LoadTileset(char TilesetID)
        {
            string dir = Application.ExecutablePath;
            dir = dir.Substring(0, dir.LastIndexOf("\\") + 1);
            Bitmap[] Tile = new Bitmap[256];
            if (File.Exists(dir + "tiles\\tileset" + TilesetID + ".png"))
            {
                Bitmap TileSet = new Bitmap(dir + "tiles\\tileset" + TilesetID + ".png");
                for (int y = 0, i = 0; y < TileSet.Height && i < 256; y += 16)
                    for (int x = 0; x < TileSet.Width && i < 256; x += 16, i++)
                    {
                        Tile[i] = new Bitmap(16, 16);
                        Graphics g = Graphics.FromImage(Tile[i]);
                        g.DrawImageUnscaledAndClipped(TileSet, new Rectangle(-x, -y, TileSet.Width, TileSet.Height));
                        g.Dispose();
                    }
                for (int i=0; i<256; i++)
                    if (Tile[i] == null)
                    {
                        Tile[i] = new Bitmap(16, 16);
                        Graphics g = Graphics.FromImage(Tile[i]);
                        g.FillRectangle(new SolidBrush(Color.Black), 0, 0, 16, 16);
                        g.Dispose();
                    }
            }
            else
            {
                for (int i = 0; i < 256; i++)
                {
                    Tile[i] = new Bitmap(16, 16);
                    Graphics g = Graphics.FromImage(Tile[i]);
                    g.FillRectangle(new SolidBrush(Color.FromArgb(i, i, i)), 0, 0, 16, 16);
                    g.Dispose();
                }
            }
            return Tile;
        }
        public static Bitmap RenderMap(byte[,] Map, Bitmap[] Tile)
        {
            Bitmap Output = new Bitmap(Map.GetLength(0)<<4,Map.GetLength(1)<<4);
            Graphics g = Graphics.FromImage(Output);
            for (int xi = 0, xl = Map.GetLength(0); xi < xl; xi++)
                for (int yi = 0, yl = Map.GetLength(1); yi < yl; yi++)
                    g.DrawImage(Tile[Map[xi, yi]], new Rectangle(xi << 4, yi << 4, 16, 16));
            g.Dispose();
            return Output;
        }
        public static Bitmap RenderMap(Bitmap[,] Map)
        {
            Bitmap Output = new Bitmap(Map.GetLength(0) << 4, Map.GetLength(1) << 4);
            Graphics g = Graphics.FromImage(Output);
            for (int x = 0, xl = Map.GetLength(0); x < xl; x++)
                for (int y = 0, yl = Map.GetLength(1); y < yl; y++)
                    g.DrawImage(Map[x, y], new Rectangle(x << 4, y << 4, 16, 16));
            g.Dispose();
            return Output;
        }
    }
}
